package OpenGL::Modern::Const;

=head1 NAME

OpenGL::Modern::Const - module encapsulating GL constants

=cut

use strict;
use warnings;

our $VERSION = '0.4001';

require XSLoader;
XSLoader::load( __PACKAGE__, $VERSION );

1;
