# This is the Perl OpenGL build configuration file.
# It contains the final OpenGL build arguments from
# the configuration process.  Access the values by
# use OpenGL::Modern::Config which defines the variable
# $OpenGL::Modern::Config containing the hash arguments needed for
# WriteMakefile()
$OpenGL::Modern::Config = {
                            'DEFINE' => '-DGLEW_NO_GLU -DGLEW_STATIC',
                            'LIBS' => '-lGL -lX11'
                          };
1;
